@@ -35,6 +35,7 @@
 #include <boost/log/keywords/auto_flush.hpp>
 #include <boost/log/keywords/rotation_size.hpp>
 #include <boost/log/keywords/time_based_rotation.hpp>
+#include <boost/log/keywords/enable_final_rotation.hpp>
 #include <boost/log/detail/config.hpp>
 #include <boost/log/detail/light_function.hpp>
 #include <boost/log/detail/parameter_tools.hpp>
@@ -387,6 +388,8 @@ public:
      *                        any size.
      * \li \c time_based_rotation - Specifies the predicate for time-based file rotation.
      *                              No time-based file rotations will be performed, if not specified.
+     * \li \c enable_final_rotation - Specifies a flag, whether or not perform log file rotation on
+     *                                sink backend destruction. By default, is \c true.
      * \li \c auto_flush - Specifies a flag, whether or not to automatically flush the file after each
      *                     written log record. By default, is \c false.
      *
@@ -472,9 +475,26 @@ public:
     BOOST_LOG_API void set_time_based_rotation(time_based_rotation_predicate const& predicate);
 
     /*!
-     * Sets the flag to automatically flush buffers of all attached streams after each log record
+     * The method allows to enable or disable log file rotation on sink destruction.
+     *
+     * By default the sink backend will rotate the log file, if it's been written to, on
+     * destruction.
+     *
+     * \param enable The flag indicates whether the final rotation should be performed.
      */
-    BOOST_LOG_API void auto_flush(bool f = true);
+    BOOST_LOG_API void enable_final_rotation(bool enable);
+
+    /*!
+     * Sets the flag to automatically flush write buffers of the file being written after each log record.
+     *
+     * \param enable The flag indicates whether the automatic buffer flush should be performed.
+     */
+    BOOST_LOG_API void auto_flush(bool enable = true);
+
+    /*!
+     * \return The name of the currently open log file. If no file is open, returns an empty path.
+     */
+    BOOST_LOG_API filesystem::path get_current_file_name() const;
 
     /*!
      * Performs scanning of the target directory for log files that may have been left from
@@ -524,7 +544,8 @@ private:
             args[keywords::open_mode | (std::ios_base::trunc | std::ios_base::out)],
             args[keywords::rotation_size | (std::numeric_limits< uintmax_t >::max)()],
             args[keywords::time_based_rotation | time_based_rotation_predicate()],
-            args[keywords::auto_flush | false]);
+            args[keywords::auto_flush | false],
+            args[keywords::enable_final_rotation | true]);
     }
     //! Constructor implementation
     BOOST_LOG_API void construct(
@@ -532,7 +553,8 @@ private:
         std::ios_base::openmode mode,
         uintmax_t rotation_size,
         time_based_rotation_predicate const& time_based_rotation,
-        bool auto_flush);
+        bool auto_flush,
+        bool enable_final_rotation);
 
     //! The method sets file name mask
     BOOST_LOG_API void set_file_name_pattern_internal(filesystem::path const& pattern);
